#!/usr/bin/env python
# -*- coding: utf-8 -*-



# Initial Relase: Ver.0.1 MareroQ 25/09/22 %03d will change the number to have 3 digits, e.g. 001, 002, 003 - 3 here is padding length.
# 				  Ver.0.2 MareroQ 01/10/22 added new type animation: "Sliding","Ripple & Sliding", "Crease"
# 				  Ver.0.3 MareroQ 02/10/22 added new type animation:"Shrink","Rotate",
#										   added new options "Reverse Frames" and efect Ping-Pong
# 				  Ver.0.4 MareroQ 04/10/22 added new type animation: "Deform","Edges on Fire","Typewriter" (using code by Tin Tran character-layers-from-text.py)

# Inspired by Pocholo plugins located here: pm_create_a_halloween_scene.py http://gimpchat.com/viewtopic.php?f=9&t=19656
#                                           pm_halloween_graveyard_scene.py http://gimpchat.com/viewtopic.php?f=9&t=20126
# 											pm_animated_dancing_ghost.py http://gimpchat.com/viewtopic.php?f=9&t=20106

# Installation:
# This plug should be placed in the user plugins folder
# Scroll bar for Python plugins https://www.gimpscripts.net/2020/10/python-fu-dialogs-window-shrink.html

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY# without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.   


from gimpfu import *
import os, sys
import math
import string
from array import array
import random

################################################# STATR ##########################################################
# character-layers-from-text.py created by Tin Tran https://gimplearn.net/viewtopic.php?f=3&t=2606 Rel 1.
def python_character_layers_from_text(image, textlayer):

	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	text = pdb.gimp_text_layer_get_text(textlayer)
	font = pdb.gimp_text_layer_get_font(textlayer)
	font_size,unit = pdb.gimp_text_layer_get_font_size(textlayer)
	offset_x,offset_y = pdb.gimp_drawable_offsets(textlayer)
	lines = text.split("\n")
	starting_y = offset_y
	for line in lines:
		textlayers = []
		totalwidth = 0
		for i in range(0,len(line)):
			char = line[i]
			if ((char == " ") and (i > 0) and (i < len(line)-1)): #if space and it's between 2 chararacters we treat it special
				before = line[i-1]
				space = line[i]
				after = line[i+1]
				special = before + space + after
				special_layer = pdb.gimp_text_fontname(image,None,0,0,special,0,TRUE,font_size,unit,font)
				before_layer = pdb.gimp_text_fontname(image,None,0,0,before,0,TRUE,font_size,unit,font)
				after_layer = pdb.gimp_text_fontname(image,None,0,0,after,0,TRUE,font_size,unit,font)
				layer_height = special_layer.height
				layer_width = special_layer.width - before_layer.width - after_layer.width
				text_layer = pdb.gimp_layer_new(image,layer_width,layer_height,RGBA_IMAGE,"Space",100,LAYER_MODE_NORMAL)
				pdb.gimp_image_insert_layer(image,text_layer,None,0)
				pdb.gimp_image_remove_layer(image,before_layer)
				pdb.gimp_image_remove_layer(image,after_layer)
				pdb.gimp_image_remove_layer(image,special_layer)
			else:
				text_layer = pdb.gimp_text_fontname(image,None,0,0,char,0,TRUE,font_size,unit,font)
			totalwidth += text_layer.width 
			textlayers.append(text_layer)

		starting_x = offset_x + (textlayer.width/2) - totalwidth/2
		for text_layer in textlayers: #loop through and move it into its placement
			pdb.gimp_layer_set_offsets(text_layer,starting_x,starting_y)
			starting_x += text_layer.width
		starting_y += text_layer.height #increment it for next round/line
	textlayer.visible = False	
	pdb.gimp_image_remove_layer(image, textlayer)
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
################################### END ##########################################################################
	
def centered(image,layer):

	wh = pdb.gimp_drawable_width(layer)
	hh = pdb.gimp_drawable_height(layer)
	w = pdb.gimp_image_width(image)
	h = pdb.gimp_image_height(image)
	pdb.gimp_image_set_active_layer(image,layer)
	centerx=w/2
	centery=h/2
	adw=wh/2
	adh=hh/2
	sw=centerx - adw
	sh=centery - adh		
	pdb.gimp_layer_set_offsets(layer, sw, sh)

################### Selection to neon glow effect by BdR based in part on Glow Rel 1 by Tin Tran (http://bakon.ca/gimplearn)#################	
def python_neonglow(image, layer, glowcolor, glowsize):

	# prepare for clean up later
	prev_layer = image.active_layer
	
	# gaussian blur sizes (example 45px, 30px 15px)
	glowsize2 = (glowsize * 2) / 3
	glowsize3 = glowsize / 3
	grow = glowsize / 15

	is_selection, x1, y1, x2, y2 = pdb.gimp_selection_bounds(image)
	
	if not (is_selection):
		pdb.gimp_message('There is nothing selected, create a selected region and try again.')
	else:
		# copy selection to new layer
		pdb.gimp_edit_copy(image.active_layer)
		glowarea = pdb.gimp_edit_paste(layer, False)
		pdb.gimp_floating_sel_to_layer(glowarea)
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,image.active_layer)
		image.active_layer.name = "Glow highlight area"

		# adds small outerglow layer
		glowlayer1 = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"outer glow",100,LAYER_MODE_NORMAL_LEGACY)
		pdb.gimp_image_insert_layer(image,glowlayer1,None,1)
		pdb.gimp_selection_grow(image,grow)
		pdb.gimp_context_set_foreground(glowcolor)
		pdb.gimp_edit_fill(glowlayer1,FOREGROUND_FILL)
		pdb.gimp_selection_none(image)
		pdb.plug_in_gauss(image,glowlayer1,glowsize,glowsize,1)
		image.active_layer.name = "Outer glow 1"
		
		#dodge outerglow copy
		glowlayer2 = pdb.gimp_layer_new_from_drawable(glowlayer1,image)
		pdb.gimp_image_insert_layer(image,glowlayer2,None,0)
		pdb.plug_in_gauss(image,glowlayer2,glowsize2,glowsize2,1)
		pdb.gimp_layer_set_mode(glowlayer2,DODGE_MODE)
		image.active_layer.name = "Outer glow 2"

		glowlayer3 = pdb.gimp_layer_new_from_drawable(glowlayer2,image)
		pdb.gimp_image_insert_layer(image,glowlayer3,None,1)
		pdb.plug_in_gauss(image,glowlayer3,glowsize3,glowsize3,1)
		pdb.gimp_layer_set_mode(glowlayer3,DODGE_MODE)
		image.active_layer.name = "Outer glow 3"
		pdb.gimp_image_raise_item_to_top(image, layer)
################################################################################################################################################
		
global apply_layers
apply_layers = []
	
def check_layers(layer):
	apply_layers.append(layer)
	
def combine(image,layer):
	pdb.gimp_image_undo_group_start(image)
	layer = image.layers[-1]
	pdb.gimp_image_set_active_layer(image,layer)
	for currentlayer in image.layers:
		check_layers(currentlayer)	
	parent = pdb.gimp_item_get_parent(layer)
	name = pdb.gimp_item_get_name(layer)
	position = pdb.gimp_image_get_item_position(image , layer)
	pdb.gimp_edit_copy(layer)
	copy = pdb.gimp_edit_paste(layer ,1)
	pdb.gimp_floating_sel_to_layer(copy)
	pdb.gimp_image_raise_item_to_top(image, copy)
	for layer in apply_layers:
		pdb.gimp_item_set_visible(layer, 1)	
		parent = pdb.gimp_item_get_parent(layer)
		position = pdb.gimp_image_get_item_position(image, layer)
		new_layer = pdb.gimp_layer_new_from_drawable(copy, image)
		pdb.gimp_image_insert_layer(image,new_layer,parent,position)
		pdb.gimp_image_raise_item(image, layer)
		pdb.gimp_image_merge_down(image, layer, EXPAND_AS_NECESSARY)
	pdb.gimp_image_remove_layer(image, copy)
	pdb.gimp_image_undo_group_end(image)


def bump_text(image, layer):

	pdb.gimp_layer_resize_to_image_size(layer)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	pdb.gimp_selection_feather(image,15)
	channel = pdb.gimp_selection_save(image)
	pdb.gimp_selection_none(image)
	azimuth = 45
	pdb.plug_in_bump_map(image,layer,channel,azimuth,27,65,0,0,0,0.5,True,False,1)

	
def MrQ_animated_halloween(image,layer,background_c,
	font_name_b, text_b, text_b_color, font_size_b, 
	font_name_a, text_a, font_size_a, text_a_color, glowcolor, glowsize,
	margin_x, margin_y, type_anim, frames,reverse,ping_pong, regex, play):

	new_image = gimp.Image(100, 100, RGB)
	
	black_layer = pdb.gimp_layer_new(new_image, 100,100, RGBA_IMAGE, "Black Layer", 100, LAYER_MODE_NORMAL_LEGACY)
	pdb.gimp_image_insert_layer(new_image, black_layer, None, -1)
	
	# create layer with text and add it to the image: image, drawable, x, y, text, border, antialias, size, size_type, fontname
	gimp.set_foreground(text_b_color)
	text_layer_b = pdb.gimp_text_fontname(new_image, None, 0, 0, text_b, 0, 1, font_size_b, PIXELS, font_name_b )
	pdb.gimp_text_layer_set_justification (text_layer_b, 2)

	gimp.set_foreground(text_a_color)		
	text_layer_g = pdb.gimp_text_fontname(new_image, None, 0, 0, text_a, 0, 1, font_size_a, PIXELS, font_name_a)
	pdb.gimp_text_layer_set_justification (text_layer_g, 2)

	pdb.gimp_image_resize_to_layers(new_image)
	pdb.gimp_image_resize(new_image, new_image.width+2*margin_x,new_image.height+2*margin_y, 0,0)
	centered(new_image,text_layer_b)
	centered(new_image,text_layer_g)
	pdb.gimp_layer_resize_to_image_size(black_layer)	
	pdb.gimp_layer_resize_to_image_size(text_layer_g)
		
	gimp.set_foreground(background_c)
	pdb.gimp_drawable_fill(black_layer, 0)
	
	bump_text(new_image,text_layer_b)
	mer=pdb.gimp_image_merge_down(new_image, text_layer_b, 0)
	pdb.gimp_item_set_visible(mer, 0)	
	bump_text(new_image,text_layer_g)	

	pdb.gimp_image_select_item(new_image, 2, text_layer_g)
	pdb.gimp_image_set_active_layer(new_image,text_layer_g)
	python_neonglow(new_image,text_layer_g, glowcolor, glowsize)# (200,20,20),30)
	layer_gm = pdb.gimp_image_merge_visible_layers(new_image, 0)
		
	if type_anim==0:

		for x in range(1,int(frames)):
			am =x*360/int(frames) # angle 0-360
			of =x*500/int(frames) # offset 0-500
			pdb.gimp_image_set_active_layer(new_image,layer_gm)
			pdb.plug_in_gmic_qt(new_image, layer_gm, 1, 1, "-ripple 20,200,2,"+ str(am) + "," + str(of))
			
	if type_anim==1 or type_anim==2:
	
		pdb.plug_in_autocrop_layer(new_image, layer_gm)
		a,b = pdb.gimp_drawable_offsets(layer_gm)
		wl = pdb.gimp_drawable_width(layer_gm)
		wI = pdb.gimp_image_width(new_image)
		pdb.gimp_layer_set_offsets(layer_gm,-wl,b)
		inc = float(2*wI/int(frames))

		x=0
		for x in range(0,int(frames)):
		
			layer_gm.visible = True
			pdb.gimp_layer_set_offsets(layer_gm, -wl + (x+1)*inc,b)
			frame_layer = pdb.gimp_layer_new_from_visible(new_image,new_image,"frame")
			pdb.gimp_image_insert_layer(new_image,frame_layer,None,0)
			frame_layer.visible = False
			
			if type_anim==2:
			
				layer_gm.visible = False			
				am =x*360/int(frames) # angle 0-360
				of =x*500/int(frames) # offset 0-500
				pdb.plug_in_gmic_qt(new_image, frame_layer, 1, 0, "-ripple 20,200,2,"+ str(am) + "," + str(of))

		pdb.gimp_layer_resize_to_image_size(new_image.layers[int(frames)])			
		pdb.gimp_image_remove_layer(new_image, new_image.layers[0])

	if type_anim==3:
		
		for x in range(1,int(frames)):
			cr =200-x*200/int(frames) # amplitude 0-200
			pdb.plug_in_gmic_qt(new_image, layer_gm, 1, 1, "fx_crease "+ str(cr) + ",100,0")
		pdb.gimp_image_raise_item_to_top(new_image, layer_gm)
		
			
	if type_anim==4: # zoom
		pdb.gimp_image_set_active_layer(new_image,layer_gm)
		for x in range(1,int(frames)):
			zm=1.000 - 1.0000*x/int(frames)
			pdb.plug_in_gmic_qt(new_image, layer_gm, 1, 1, "fx_zoom "+ str(zm) + ",50,50,0")			
		
	if type_anim==5: # rotate	
		rotate_angle=0
		w = pdb.gimp_drawable_width(layer_gm)
		h = pdb.gimp_drawable_height(layer_gm)		
		for x in range(1,int(frames)):
			inc_angle=360/frames
			rotate_angle=rotate_angle + inc_angle
			pdb.gimp_image_get_active_layer(new_image)
			copy = pdb.gimp_layer_copy(layer_gm, TRUE)
			pdb.gimp_image_insert_layer(new_image, copy, None, -1)
			pdb.gimp_item_transform_rotate(copy, rotate_angle*0.01745329252 ,FALSE, w/2, h/2)
			pdb.gimp_layer_resize_to_image_size(copy)

	if type_anim==6:
		copy = pdb.gimp_layer_copy(layer_gm, TRUE)
		pdb.gimp_image_remove_layer(new_image, layer_gm)
		
		gimp.set_foreground(text_a_color)
		text_layer_g = pdb.gimp_text_fontname(new_image, None, 0, 0, text_a, 0, 1, font_size_a, PIXELS, font_name_a)
		pdb.gimp_text_layer_set_justification (text_layer_g, 2)
		centered(new_image,text_layer_g)
		python_character_layers_from_text(new_image,text_layer_g)
		for layer in new_image.layers:
			bump_text(new_image,layer)
			pdb.gimp_item_set_visible(layer, 0)
			pdb.gimp_image_set_active_layer(new_image,layer)
			pdb.gimp_image_raise_item_to_top(new_image, layer)

		pdb.gimp_image_lower_item_to_bottom(new_image,new_image.layers[0])	
		for layer in new_image.layers:
			pdb.gimp_item_set_visible(layer, 1)
			frame_layer = pdb.gimp_layer_new_from_visible(new_image,new_image,"frame")
			pdb.gimp_image_insert_layer(new_image,frame_layer,None,0)
			pdb.gimp_image_lower_item_to_bottom(new_image,layer)
			pdb.gimp_image_lower_item_to_bottom(new_image,frame_layer)
			pdb.gimp_image_merge_down(new_image, layer, 0)
			
		for layer in new_image.layers:
			pdb.gimp_image_set_active_layer(new_image,layer)
			pdb.gimp_image_raise_item_to_top(new_image, layer)
		pdb.gimp_image_set_active_layer(new_image,new_image.layers[0])
		pdb.gimp_image_lower_item_to_bottom(new_image,new_image.layers[0])
		pdb.gimp_image_insert_layer(new_image, copy, None, 0)
		pdb.gimp_image_raise_item_to_top(new_image, copy)
		combine(new_image,new_image.layers[-1])
		
	if type_anim==7:
		pdb.gimp_image_select_item(new_image,CHANNEL_OP_REPLACE,layer_gm)
		pdb.gimp_image_set_active_layer(new_image,layer_gm)
		pdb.plug_in_gmic_qt(new_image, layer_gm, 1, 1, "fx_fire_edges 0.7,0.25,0.5,25,"+ str(int(frames)) +",10,10,0,50,50")
		pdb.gimp_selection_none(new_image)
		pdb.gimp_image_remove_layer(new_image,layer_gm)

	if type_anim==8:
		for x in range(1,int(frames)):
			of =x*10.0/int(frames) 
			pdb.gimp_image_set_active_layer(new_image,layer_gm)
			pdb.plug_in_gmic_qt(new_image, layer_gm, 1, 1, "deform " +str(of))
			
	if type_anim!=6:			
		pdb.gimp_image_set_active_layer(new_image,mer)
		pdb.gimp_item_set_visible(mer, 1)
		combine(new_image,mer)
		
	if type_anim!=3 or type_anim!=6:
		pdb.gimp_image_remove_layer(new_image, new_image.layers[-1])

	if reverse:
		for layer in new_image.layers:
			pdb.gimp_image_set_active_layer(new_image,layer)
			pdb.gimp_image_raise_item_to_top(new_image, layer)
			
	if ping_pong:
		for layer in new_image.layers:
			pdb.gimp_image_set_active_layer(new_image,layer)
			copy = pdb.gimp_layer_copy(layer, TRUE)
			pdb.gimp_image_insert_layer(new_image, copy, None, -1)			
			pdb.gimp_image_raise_item_to_top(new_image, layer)
			
# Rename ##########################################################################	
	pdb.gimp_image_undo_group_start(new_image)
	gimp.progress_init("Renaming layers")
	i = 1
	for l in reversed(new_image.layers):
		l.name = (regex) % i
		i += 1
	pdb.gimp_image_undo_group_end(new_image)
###################################################################################

	gimp.Display(new_image)
	
	#Play animation	
	if play:
		#pdb.plug_in_animationplay(new_image, layer) bug
		pdb.plug_in_animationplay(new_image, None)
	   
register(
	"MrQ_animated_halloween",
	" Create a halloween animated scene. Ver.0.4.\n New Image Size = Max Text Size + Margin.",
	"Create a halloween animated scene.\nRecommended use font Trick or Treat (free for non-commercial uses).",
	"MareroQ",
	"GPL",
	"2022",
	"Halloween animated scene...",
	"*",
	[
		(PF_IMAGE,    "image", "Input image", None),
		(PF_DRAWABLE, "layer", "Input layer", None),
		(PF_COLOR,    "background_c", "Background Color", (0,0,0)),		
		
		(PF_FONT,     "font_name", "Back Font Name:", "Trick or Treat BV"),
		(PF_TEXT,     "text_b", "Back Text :", "Q"),
		(PF_COLOR,    "text_b_color", "Back Color", (128,128,128)),
		(PF_SPINNER,  "font_size_b", "Back Font Size:",  500, (10, 1000, 1)),		

		(PF_FONT,     "font_name_a", "Front Font Name:", "Trick or Treat BV"),		
		(PF_TEXT,     "text_a", "Front Text:", "O"),		
		(PF_SPINNER,  "font_size", "Front Font Size:",  300, (10, 1000, 1)),	
		(PF_COLOR,    "text_a_color", "Front Text Color", (10,100,200)),
		
		(PF_COLOR,    "glowcolor", "Front Glow Color:", (80,200,200)),
		(PF_SPINNER,  "glowsize", "Front Glow Size:", 50, (0, 500, 1)),
		
		(PF_SPINNER,  "margin_x", "Margin X:", 50, (1, 500, 1)),
		(PF_SPINNER,  "margin_y", "Margin Y:", 10, (1, 500, 1)),
		
		(PF_OPTION,   "type_anim",   "Type of Animation:", 0, ["Ripple","Sliding","Ripple & Sliding", "Crease","Shrink","Rotate", "Typewriter","Edges on Fire","Deform"]),		
		(PF_SPINNER,  "frames", "Frames:", 10, (5, 100, 1)),
		(PF_TOGGLE,   "reverse",   "Reverse Frames?", 0),		
		(PF_TOGGLE,   "ping_pong",   "Ping-Pong?", 0),		
		(PF_STRING,   "regex", "Rename Layers:", "Halloween %03d (50ms)(replace)"),
		(PF_TOGGLE,   "play",   "Run Animation Playback?", 0)		
	],
	[],
	MrQ_animated_halloween, menu="<Image>/Filters/Animation/Animators",
	domain=("gimp20", gimp.locale_directory))
main()							